
def mul_polynomial_by_binomial(acoefs, b):
   alen = len(acoefs)
   midcoefs = [acoefs[j] * b + acoefs[j - 1] for j in range(1, alen)]
   return [b * acoefs[0]] + midcoefs + [acoefs[alen - 1]]

def elem_pols(t):
   m = len(t)
   result = [1]
   for j in range(m):
      result = mul_polynomial_by_binomial(result, t[j])
   return list(reversed(result))

def prod_dif(x, j):
   return prod(x[j] - x[k] for k in range(j) + range(j + 1, len(x)))

def hom_via_geom_progr(x, m):
   n = len(x)
   result = 0 * x[0]
   for j in range(n):
      result += x[j] ** (n + m - 1) / prod_dif(x, j) 
   return result

def powersum(x, m):
   n = len(x)
   return sum(x[j] ** m for j in range(n))

# test recursive formulas

def random_rational_list(n):
   s = set()
   while len(s) < n:
      q = QQ.random_element()
      s.add(q)
   return list(s)

def test_all_elem_recursive_numer(n, verbos):
   if verbos >= 2:
      print('test_all_elem_recursive_numer, n = %d' % n)
   F = QQ
   x = random_rational_list(n + 1)
   xless = x[ : n]
   epols_many_vars = elem_pols(x)
   lhs = epols_many_vars
   epols_less_vars = elem_pols(xless)
   # epols_many_vars[j + 1] = epols_less_vars[j + 1] + x[n] * epols_less_vars[j]
   rhs = elem_pols(xless) + [F.zero()]
   for j in range(n + 1):
      rhs[j + 1] += x[n] * epols_less_vars[j]
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return epols_many_vars == rhs

def big_test_elem_recursive_numer(nmax, nrep, verbos):
   if verbos >= 1:
      print('big_test_elem_recursive_numer, nmax = %d, nrep = %d' % (nmax, nrep))
   tasks = [(n, r) for n in range(1, nmax + 1) for r in range(nrep)]
   return all(test_all_elem_recursive_numer(n, verbos) for (n, r) in tasks)

def test_hom_recursive_numer(n, m, verbos):
   if verbos >= 2:
      print('test_hom_recursive_numer, n = %d, m = %d' % (n, m))
   F = QQ
   x = random_rational_list(n + 1)
   xless = x[ : n]
   # h_m(xless, xlast) = h_m(xless) + xlast * h_{m-1}(xless, xlast)
   lhs = hom_via_geom_progr(x, m)
   last_factor = hom_via_geom_progr(x, m - 1) if m >= 1 else F.zero()
   rhs = hom_via_geom_progr(xless, m) + x[n] * last_factor
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_hom_recursive_numer(nmax, mmax, nrep, verbos):
   if verbos >= 1:
      print('big_test_hom_recursive_numer, nmax = %d, mmax = %d, nrep = %d' % (nmax, mmax, nrep))
   tasks = [(n, m, r) for n in range(1, nmax + 1) for m in range(mmax + 1) for r in range(nrep)]
   return all(test_hom_recursive_numer(n, m, verbos) for (n, m, r) in tasks)

def test_powersum_recursive_numer(n, m, verbos):
   if verbos >= 2:
      print('test_powersum_recursive_numer, n = %d, m = %d' % (n, m))
   F = QQ
   x = random_rational_list(n + 1)
   xless = x[ : n]
   lhs = powersum(x, m)
   rhs = powersum(xless, m) + x[n] ** m
   if verbos >= 3:
      print('lhs = ' + str(lhs))
      print('rhs = ' + str(rhs))
   return lhs == rhs

def big_test_powersum_recursive_numer(nmax, mmax, nrep, verbos):
   if verbos >= 1:
      print('big_test_powersum_recursive_numer, nmax = %d, mmax = %d, nrep = %d' % (nmax, mmax, nrep))
   tasks = [(n, m, r) for n in range(1, nmax + 1) for m in range(mmax + 1) for r in range(nrep)]
   return all(test_powersum_recursive_numer(n, m, verbos) for (n, m, r) in tasks)

import time

def all_tests_elem_hom_powersum_recursive_numer(nmax, nrep):
   mmax = 2 * nmax
   t = time.time()
   verbos = 2
   result_elem = big_test_elem_recursive_numer(nmax, nrep, verbos)
   result_hom = big_test_hom_recursive_numer(nmax, mmax, nrep, verbos)
   result_powersum = big_test_powersum_recursive_numer(nmax, mmax, nrep, verbos)
   total_result = result_elem and result_hom and result_powersum
   t = time.time() - t
   print('total result = ' + str(total_result))
   print('time = %.2f seconds' % t)

# with nmax = 20, nrep = 10, time < 60 sec
all_tests_elem_hom_powersum_recursive_numer(20, 10)

